/*

   generate periodic lattice from unitcell,  Jan02 JS
   output as daten.def or povray file

   v1.0 initial version 3D, based on pic2.cc, m_sc.c, cubic.c, edgevec.c
   
   compile: g++ -Wall -o a lattice.c -lm
   use:     a.out [options] e00 e01 e10 e11
   use:     a.out [options] e00 e01 e02 e10 e11 e12 e20 e21 e22
   options:
     -l<n>   lattice number (not used)
     -t<n>   output type, 0=def, 1=povray (not used)
     -d<n>   lattice dimension (not needed)
     -a      use a cubic base (ignore definition)
   examples:
     a.out     2 1  1 2	          # volume=Det((2,1),(1,2))=3
     a.out     2 0  0 2 >a.def    # volume=4
     a.out -t1 2 0  0 2 >a.pov
     nice -30 povray +FPPM +L/usr/lib/povray31/include\
       +L/usr/lib/povray3/include +W500 +H375 +K0 +Oa.ppm +Ia.pov +A
     # ppmtopgm a.ppm | 
     pnmtops -rle -noturn a.ppm >a.eps
  
  ToDo:
    - animation of rotating or translations (til4->kago->til9 ...)
    - wire frame 3D viewer (Drahtgitter 3D Anzeige, => magnon.tcl)
    - print only some layers (example: [1,-1,1] layer)
      and the rest half transparent (opaque) => video?

  remarks:   
   edgevectors mcf:  1,0,0,1,  1,1,1,-1,  2,1,1,2,  2,0,0,2,  2,1,-1,2,  
               utlf: 1,0,0,1,             3,0,2,1,  2,0,0,2,
               (most compact form, upper triangular lattice form)

   read unitcell from file

 */

#include <stdlib.h>
#include <stdio.h>

#define Layer1m11

typedef struct {
  int type;      /* type of site (0=data-stop) */
  double pos[3]; /* coordinates of the site */
} site_t;

typedef struct {
  int type;      /* type of bond (0=data-stop) */
  int s1,s2;     /* bonded sites */
  int r2[3];     /* relativ coords of site 2 */
  double vec[3]; /* direction of bonds ((pos[s2]+u2*a)-pos[s1]) */
  int w;         /* phase index (magnetic fields) = vec-idx */
} bond_t;

/*
   square lattices: x^2+y^2=V            V=1,2,4,5,8,9,10,...
   cubic lattices: (x^2+y^2+z^2)^3=V^2   V=1,64,...
*/
/* --- example: squared unitcell with 4 sites and 6 bonds ---
      +-----------------+
      |        |        |
      |        0        |      ^ a1=(0,1)
      |      /   \      |      |
      |--- 1       2 ---|      |
      |      \   /      |      |
      |        3        |      |
      |        |        |      --------> a0=(1,0)
      +-----------------+
*/
 double til11_xa[3][3]={{1,0,0},{0,1,0},{0,0,1}}; /* base vectors of unitcell */
 site_t til11_xs[]={
  { 1, {0.5 ,0.75,0}}, /* site 0 */
  { 1, {0.25,0.5 ,0}}, /* site 1 */
  { 1, {0.75,0.5 ,0}}, /* site 2 */
  { 1, {0.5 ,0.25,0}}, /* site 3 */
  { 0, {0   ,0   ,0}}};
 bond_t til11_xb[]={
  { 1,  0, 1, {0,0,0}, {-0.25,-0.25,0}, 0}, /* intra coupling */
  { 1,  0, 2, {0,0,0}, { 0.25,-0.25,0}, 1},
  { 1,  1, 3, {0,0,0}, { 0.25,-0.25,0}, 1},
  { 1,  2, 3, {0,0,0}, {-0.25,-0.25,0}, 0},
  { 2,  0, 3, {0,1,0}, { 0   , 0.5 ,0}, 2}, /* inter coupling */
  { 2,  2, 1, {1,0,0}, { 0.5 , 0   ,0}, 3},
  { 0,  0, 0, {0,0,0}, { 0   , 0   ,0}, 0}};

#define q3 1.732050808  /* suare-root of 3 */
/* --- kagome unitcell with 3 sites and 6 bonds ---
      +--------1--------+
      |        | \      |
      |        |   \    |      ^ a1=(cos(60)=1/2,sin(60)=0.866=q3/2)
      |        |     \  |      |
      2 ------ 0 ------ 2      |
      | \      |        |      |
      |   \    |        |      |
      |     \  |        |      --------> a0=(1,0)
      +--------1--------+
*/
 double til8_xa[3][3]={{1,0,0},{0.5,0.5*q3,0},{0,0,1}}; /* base vectors of unitcell */
 site_t til8_xs[]={
  { 1, {0.5 ,0.5 ,0}}, /* site 0 */
  { 1, {0.5 ,0   ,0}}, /* site 1 */
  { 1, {0   ,0.5 ,0}}, /* site 2 */
  { 0, {0   ,0   ,0}}};
 bond_t til8_xb[]={
  { 1,  1, 0, { 0, 0, 0}, { 0   , 0.5 ,0}, 0}, /*  60deg */
  { 1,  0, 1, { 0, 1, 0}, { 0   , 0.5 ,0}, 0},
  { 1,  2, 0, { 0, 0, 0}, { 0.5 , 0   ,0}, 1}, /*   0deg */
  { 1,  0, 2, { 1, 0, 0}, { 0.5 , 0   ,0}, 1},
  { 1,  1, 2, { 0, 0, 0}, {-0.5 , 0.5 ,0}, 2}, /* 120deg */
  { 1,  2, 1, {-1, 1, 0}, {-0.5 , 0.5 ,0}, 2},
  { 0,  0, 0, {0,0,0}, { 0   , 0   ,0}, 0}};

/* --- cm0103231
  two interpenetrating pyrochlore sublattices with half unitcell
  cubic unit cell 2x2x2 N=4 Nb=3*4=2*6=12 (similar to BCC-lattice)
*/
 double pyr_xa[3][3]={{1,0,0},{0,1,0},{0,0,1}}; /* wrong? */
 site_t pyr_xs[]={
  { 1, {0   ,0   ,0  }}, /* site 0 */
  { 1, {0.5 ,0.5 ,0  }}, /* site 1 */
  { 1, {0.5 ,0   ,0.5}}, /* site 2 */
  { 1, {0   ,0.5 ,0.5}}, /* site 3 */
  { 0, {0   ,0   ,0}}};
 bond_t pyr_xb[]={
  { 1,  0, 1, { 0, 0, 0}, { 0.5 , 0.5 , 0  }, 0}, /* tetraeder 1 */
  { 1,  0, 2, { 0, 0, 0}, { 0.5 , 0   , 0.5}, 0}, /* phase not set */
  { 1,  0, 3, { 0, 0, 0}, { 0   , 0.5 , 0.5}, 0},
  { 1,  1, 2, { 0, 0, 0}, { 0   ,-0.5 , 0.5}, 0},
  { 1,  1, 3, { 0, 0, 0}, {-0.5 , 0   , 0.5}, 0},
  { 1,  2, 3, { 0, 0, 0}, {-0.5 , 0.5 , 0  }, 0},
  { 2,  0, 1, {-1,-1, 0}, {-0.5 ,-0.5 , 0  }, 0}, /* tetraeder 2 */
  { 2,  0, 2, {-1, 0,-1}, {-0.5 , 0   ,-0.5}, 0}, /* wrong vecs ? */
  { 2,  0, 3, { 0,-1,-1}, { 0   ,-0.5 ,-0.5}, 0},
  { 2,  1, 2, { 0, 1,-1}, { 0   , 0.5 ,-0.5}, 0},
  { 2,  1, 3, { 1, 0,-1}, { 0.5 , 0   ,-0.5}, 0},
  { 2,  2, 3, { 1,-1, 0}, { 0.5 ,-0.5 , 0  }, 0},
  { 0,  0, 0, { 0, 0, 0}, { 0   , 0   , 0  }, 0}};


#define Ns 1000  /* maximum number of sites */
#define Nb 8000  /* maximum number of bonds */

int dim=2, nn=0;
char *title="no title";

int    xe[3][3]={{1,0,0},{0,1,0},{0,0,1}}; /* vol=1 */
// int xe[3][3]={{0,1,1},{1,0,1},{1,1,0}}; /* vol=2 */
// int xe[3][3]={{2,1,1},{1,2,1},{1,1,2}}; /* vol=4 */
/* unitcell specifique: */
double (*xa)[3] = til11_xa; /* = (double (*)[3])til11_xa; */
site_t *xs      = til11_xs;
bond_t *xb      = til11_xb;
/* full cluster */
site_t *ys      = NULL;
bond_t *yb      = NULL;

/* see edgevec.c */
/* 2D-rotations det(T)=+1 oder als Permutation von sites tabellieren? */
int rotate[2][3][3]={
  {{1, 0, 0}, { 0, 1, 0}, { 0, 0, 1}},
  {{0, 1, 0}, {-1, 0, 0}, { 0, 0, 1}},
};         
/* 2D-reflections det(T)=+1 oder als Vectoren \pm 1 */
int reflect[2][3][3]={
 {{ 1, 0, 0}, { 0, 1, 0}, { 0, 0, 1}},
 {{-1, 0, 0}, { 0,-1, 0}, { 0, 0, 1}},
};         


int    get_deti(int    e[3][3]){   /* volume=det(a) */
 return (
     e[0][0]*(e[1][1]*e[2][2]-e[1][2]*e[2][1])   /* 00(11 22 - 12 21) */ 
    +e[0][1]*(e[1][2]*e[2][0]-e[1][0]*e[2][2])   /* 01(12 20 - 10 22) */
    +e[0][2]*(e[1][0]*e[2][1]-e[1][1]*e[2][0])); /* 02(10 21 - 11 20) */
}

double get_detd(double e[3][3]){   /* volume=det(a) */
 return (
     e[0][0]*(e[1][1]*e[2][2]-e[1][2]*e[2][1])   /* 00(11 22 - 12 21) */ 
    +e[0][1]*(e[1][2]*e[2][0]-e[1][0]*e[2][2])   /* 01(12 20 - 10 22) */
    +e[0][2]*(e[1][0]*e[2][1]-e[1][1]*e[2][0])); /* 02(10 21 - 11 20) */
}

int sprod(int a[3], int b[3]){  /* spat-product */
   return a[0]*b[0]+a[1]*b[1]+a[2]*b[2]; }

/* y = x1 + a * x2  */
double *add( double x1[3], double x2[3], double a, double y[3]){
   int i;
   for (i=0;i<3;i++) y[i] = x1[i] + a*x2[i];
   return y;
}

/* y = matrix_a * x  */
double *mltply( double a[3][3], double x[3], double y[3]){
   int i;
   if (x==y) { fprintf(stderr,"&y==&x"); exit(1); }
   for (i=0;i<3;i++) 
     y[i] = a[0][i]*x[0] + a[1][i]*x[1] + a[2][i]*x[2];
   return y;
}

/* better return *[3][3] (*i_matrix_t) */
int product(int a[3][3],int u[3][3],int au[3][3]){ /* au=a*u */
   int i,j,b[3][3]={{0,0,0},{0,0,0},{0,0,0}}; /* buffer if &au=&a */
   for (i=0;i<3;i++) for (j=0;j<3;j++)
     b[i][j]  = a[i][0]*u[0][j] + a[i][1]*u[1][j] + a[i][2]*u[2][j];
   for (i=0;i<3;i++) for (j=0;j<3;j++)
     au[i][j] = b[i][j];
   return 0;
}

int dotprod(int a[3][3],int b[3][3],int ab[3][3]){ /* ab=a.b */
   int i,j;
   for (i=0;i<3;i++)
   for (j=0;j<3;j++) ab[i][j] = sprod(a[i],b[j]);
   return 0;
}

/* TransponiertAdjuncte=det(A)*A^(-1); -> Bildung der Inversen Matrix */
int adjunct(int a[3][3],int adjtrans[3][3]){
   if(adjtrans==a){ fprintf(stderr,"---error1---\n");exit(1); }
   adjtrans[0][0]=a[1][1]*a[2][2]-a[1][2]*a[2][1];
   adjtrans[0][1]=a[2][1]*a[0][2]-a[2][2]*a[0][1];
   adjtrans[0][2]=a[0][1]*a[1][2]-a[0][2]*a[1][1];
   adjtrans[1][0]=a[1][2]*a[2][0]-a[1][0]*a[2][2];
   adjtrans[1][1]=a[2][2]*a[0][0]-a[2][0]*a[0][2];
   adjtrans[1][2]=a[0][2]*a[1][0]-a[0][0]*a[1][2];
   adjtrans[2][0]=a[1][0]*a[2][1]-a[1][1]*a[2][0];
   adjtrans[2][1]=a[2][0]*a[0][1]-a[2][1]*a[0][0];
   adjtrans[2][2]=a[0][0]*a[1][1]-a[0][1]*a[1][0];
   return 0;
}

int compare(int a1[3][3], int a2[3][3]){ /* 2x2 */
    int bb1[3][3],bb2[3][3],i1,i2,v;
    /* compare  A1*T_i*A2^-1=int-matrix? */
    /* rechts - links Reihenfolge beachten! */
    v=abs(get_deti(a2));
    for(i1=0;i1<2*2;i1++){	/* all rotations+reflections */
       adjunct(a2,bb1);
       product( rotate[i1/2],bb1,bb2);
       product(reflect[i1%2],bb2,bb1);
       product(  a1,         bb1,bb2);
       for(i2=0;i2<2*2;i2++)if(abs(bb2[i2/2][i2%2])%(v)!=0)break;
       if(i2>=2*2){
         fprintf(stderr,"# V=%2u ROT=%d REF=%d U=(%d %d, %d %d)\n",
           v,i1/2,i1%2,
           bb2[0][0]/v,bb2[0][1]/v,
           bb2[1][0]/v,bb2[1][1]/v);
         break;
       }   
    }      
    if(i1==2*2) { /* fprintf(stderr,"# a1!=a2\n"); */ return -1;}
    return 0;
}

int minmax(int e[3][3],int emin[3],int emax[3]){
   /* einhuellenden Kubus ermitteln (alle eck-koords checken) */
   int i,j,k;
   for(i=0;i<3;i++){                     
     emin[i]=emax[i]=e[0][i]+e[1][i];    
     if(emin[i]>0)emin[i]=0;
     if(emax[i]<0)emax[i]=0;
     for(j=0;j<3;j++){
       if(e[j][i]<emin[i])emin[i]=e[j][i];
       if(e[j][i]>emax[i])emax[i]=e[j][i];
       for(k=j+1;k<3;k++){
         if(e[j][i]+e[k][i]<emin[i])emin[i]=e[j][i]+e[k][i];
         if(e[j][i]+e[k][i]>emax[i])emax[i]=e[j][i]+e[k][i];
   }}}
#if 0                                   
   printf("# emin=(%d %d %d) emax=(%d %d %d)\n", 
          emin[0],emin[1],emin[2],emax[0],emax[1],emax[2]);
#endif
   return 0;
}

/* --- from m_sc.c for simple cubic lattice --- */
#define  NN       999	/* max. number of vertices (unitcells) */
int vertex[NN][3];	/* unitcell-position = (x,y,z) (same as SC) */
#define F0(txt)	{ fprintf(stderr,"\nERROR L%d: "txt"\n",__LINE__);exit(1); }

/* ================== 3D-Gitter nach Betts ===========================  
   -> utils/cubes.c 
   Berechnung kubischer Gitter aus Vektoren die Elementarzelle aufspannen 

   unit cells automatisch erzeugen durchprobieren mit l_max=N, e1<e2<e3

+++TeX+++ ---------------------------------------------------------------

 Beispiel in zwei Dimensionen. $a=(a_x,a_y)$ und $b=(b_x,b_y)$ sind die 
 zwei Vektoren, die die Elementarzelle aufspannen. 
 Sie sind im allgemeinen nicht senkrecht zueinander. 
 Der Vektor $r=(r_x,r_y)$ liegt innerhalb der Elementarzelle, wenn seine
 Komponenten $(r_a,r_b)$ bezueglich der Elementarvektoren zwischen $0$ und $1$
 liegen.
 \begin{eqnarray*}
      r &=& r_a a   + r_b b \\
 %
    a r &=& r_a a^2 + r_b a b \\
    b r &=& r_a a b + r_b b^2 \\
 %
    r_a &=& \frac{(b^2a-(a,b)b)r}{a^2b^2-(a,b)^2} \\
    r_b &=& \frac{(a^2b-(a,b)a)r}{a^2b^2-(a,b)^2} \\
 \end{eqnarray*}
 Grenzfall $(a,b)=0$ liefert bekannte Beziehungen. 
 Im 3 dimensionalen muss entsprechend ein Gleichungssystem 3.\ Ordnung 
 geloest werden.
 \begin{verbatim}
  maple> solve( { y0=ra*aa+rb*ab+rc*ac,\ 
  maple>          y1=ra*ab+rb*bb+rc*bc,\
  maple>          y2=ra*ac+rb*bc+rc*cc},{ra,rb,rc});
  {ra= ( + (bb cc - bc bc) y0 + (ac bc - cc ab) y1 + (bc ab - ac bb) y2)/%1,
   rb=-( + (ab cc - bc ac) y0 + (ac ac - cc aa) y1 + (bc aa - ac ab) y2)/%1,
   rc= ( + (bc ab - bb ac) y0 + (ab ac - aa bc) y1 + (aa bb - ab ab) y2)/%1}
  %1 := + ac bc ab + ac bc ab + bb cc aa - bc bc aa - ac ac bb - ab ab cc
  Matrixform: x=A*r; r=Adj(A)*r/Det(A)
  \end{verbatim}
  Auf die Division durch die Determinante kann evl. verzichtet werden, in dem
  die Einheit auf Det(a,b,c) gesetzt wird.
  Hilfreich koennte die Speicherung der Determinante der Einheitszelle und
  der inversen Elementarmzelle (normiert mit der Determinante) sein.
   ---TeX--- ----------------------------------------------------- */

/* vertex -> (x,y,z) */
int get_koord(const int i,int s[3]){
  if(i<0 || i>=nn) { fprintf(stderr,"error: get_koord\n"); return -1; }
  s[0]=vertex[i][0];
  s[1]=vertex[i][1];
  s[2]=vertex[i][2];
  return 0;}
  
/* (x,y,z) -> vertex */  
int get_vertex(int xx[3]){
  int i,x[3],(*e)[3]=xe,aa[3][3],bb[3][3],ry[3],ra[3],det,(*s)[3]=vertex;
  /* ra = adjunct(aa)/det(aa) * ry = aa^(-1) * ry */
  dotprod(e,e,aa); /* aa=e.e */
  det=get_deti(aa); adjunct(aa,bb);
  for(i=0;i<3;i++) ry[i]=sprod(e[i],xx);
  /* ra = adjunct(aa) * ry */
  for(i=0;i<3;i++) ra[i]= bb[i][0]*ry[0] + bb[i][1]*ry[1] + bb[i][2]*ry[2];
  for(i=0;i<3;i++) if(ra[i]<0)ra[i]=-((-ra[i]-1)/det)-1;else ra[i]/=det;
  for(i=0;i<3;i++) x[i]=xx[i]-ra[0]*e[0][i]+ra[1]*e[1][i]+ra[2]*e[2][i];
  for(i=0;i<nn;i++) if(x[0]==s[i][0] && x[1]==s[i][1] && x[2]==s[i][2]) return i;
  fprintf(stderr,"error get_vertex: %2d %2d %2d (%2d %2d %2d) => %2d %2d %2d not found\n",
    xx[0],xx[1],xx[2],ra[0],ra[1],ra[2],x[0],x[1],x[2]);
  return -1;}

int q2(int x){return x*x;}

int vertex_init(){ /* vertex_init */
   int ii,i,r[3],ra[3],aa[3][3],bb[3][3],ry[3],det,emin[3],emax[3],
     (*s)[3]=vertex, (*e)[3]=xe /* ,*a=e[0],*b=e[1],*c=e[2] */ ;
   /* plaetze finden! */
   
   /* einhuellenden Kubus ermitteln (alle eck-koords checken) */
   minmax(e,emin,emax);
   
   ii=0;
   /* ra = adjunct(aa)/det(aa) * ry = aa^(-1) * ry */
   dotprod(e,e,aa); /* aa=e.e */
   det=get_deti(aa); adjunct(aa,bb);
   for(r[0]=emin[0];r[0]<=emax[0];r[0]++)
   for(r[1]=emin[1];r[1]<=emax[1];r[1]++)  /* ineffektiv! */
   for(r[2]=emin[2];r[2]<=emax[2];r[2]++){
    for(i=0;i<3;i++) ry[i]=sprod(e[i],r);
    /* ra = adjunct(aa) * ry */
    for(i=0;i<3;i++) ra[i]= bb[i][0]*ry[0] + bb[i][1]*ry[1] + bb[i][2]*ry[2];
    if(ra[0]>=0 && ra[1]>=0 && ra[2]>=0 && ra[0]<det && ra[1]<det && ra[2]<det){
      s[ii][0]=r[0];
      s[ii][1]=r[1];
      s[ii][2]=r[2]; ii++; }
    else /* printf("\n") */ ;
   }
   if(ii!=nn)fprintf(stderr,"error vertex_init i!=N i=%d N=%d\n",ii,nn);
   /* --- now s=vertex is initialized --- */
   return 0;
}


/*  --------------------------
 test if R lying in a parallel epiped, move r to x (by n*e)
 */
int inside(int r[3],int e[3][3],int x[3]){  /* 2x2 */
   int i,aa[3][3],bb[3][3],det,ra[3],ry[3],ret=0;
   /* ra = adjunct(aa)/det(aa) * ry = aa^(-1) * ry */
   dotprod(e,e,aa); /* aa=e.e */
   det=get_deti(aa); adjunct(aa,bb);
   for(i=0;i<3;i++) ry[i]=sprod(e[i],r);
   /* ra = adjunct(aa) * ry */
   for(i=0;i<3;i++) ra[i]= bb[i][0]*ry[0] + bb[i][1]*ry[1] + bb[i][2]*ry[2];
   if(ra[0]>=0 && ra[1]>=0 && ra[2]>=0 && ra[0]<det && ra[1]<det && ra[2]<det) ret=1;
   /* ret=1: ok, its lying inside */
   /* else R is outside of E */
   if(x){
     for(i=0;i<3;i++) x[i]=r[i];
     for(i=0;i<3;i++) if(ra[i]<0)ra[i]=-((-ra[i]-1)/det)-1;else ra[i]/=det;
     for(i=0;i<3;i++) x[i]-=ra[0]*e[0][i]+ra[1]*e[1][i]+ra[2]*e[2][i];           
   }
#if 0
   printf("# inside  r(%p)= % d % d % d rc=%d\n",r,r[0],r[1],r[2],ret);
   if(x) printf("# inside rr(%p)= % d % d % d\n",      x,x[0],x[1],x[2]);
#endif
   return ret;
}

/* ---------------------------------------------------------------
   xa=base_vectors unit_cell
   xe=edge_vectors
   xs=sites_of_unitcell
   xb=bonds_of_unitcell  { type, site1, site2, r2[3], ... }
   ns=sites_per_unitcell
   nb=bonds_per_unitcell
   ex: ( 2, 1) v=5    idx <=> (x,y,z) in xa-units via xe -> convert-fkt
       (-1, 2)          0              0 1 2 3 0
       0                2 0  (2,1)     4 5 6 7 4
   0 1 2              0 1    (1,2)     0 1 2 3 0  (4,0)
     3 4 0              0                         (0,2)
     0
   numbering is not best (see upper picture for a better variant)
   --------------------------------------------------------------- */

int ini_y(double xa[3][3], int xe[3][3], site_t *xs, bond_t *xb){
    int i,j,k,n,m,ns,nb,v,r[3],rr[3];

    v=get_deti(xe);
    for(ns=0;ns<Ns;ns++) if (!xs[ns].type) break; /* num of sites */
    for(nb=0;nb<Nb;nb++) if (!xb[nb].type) break; /* num of bonds */
    ys=(site_t *)malloc(sizeof(site_t)*(ns*v+1));
    yb=(bond_t *)malloc(sizeof(bond_t)*(nb*v+1));
    if((!ys) || (!yb)) { fprintf(stderr,"malloc failed\n"); exit(1); }
    for(i=0; i<v; i++){ /* all unitcells */
     get_koord(i,r);
     for (n=0; n<ns; n++) {
       j=n+ns*i;
       ys[j].type   = xs[n].type;
//     ys[j].pos[0] = r[0]*xa[0][0] + r[1]*xa[1][0] + r[2]*xa[2][0] + xs[n].pos[0];
//     ys[j].pos[1] = r[0]*xa[0][1] + r[1]*xa[1][1] + r[2]*xa[2][1] + xs[n].pos[1];
//     ys[j].pos[2] = r[0]*xa[0][2] + r[1]*xa[1][2] + r[2]*xa[2][2] + xs[n].pos[2];
       ys[j].pos[0] = r[0] + xs[n].pos[0];
       ys[j].pos[1] = r[1] + xs[n].pos[1];
       ys[j].pos[2] = r[2] + xs[n].pos[2];
     }
    }
    ys[ns*v].type=0;

    for(i=0; i<v; i++){ /* all unitcells */
     get_koord(i,r);
     for (n=0; n<nb; n++) {
      rr[0]=r[0]+xb[n].r2[0];
      rr[1]=r[1]+xb[n].r2[1];
      rr[2]=r[2]+xb[n].r2[2];   /* neighbourcell */
      j=i; k=1;
      if( xb[n].r2[0] || xb[n].r2[1] || xb[n].r2[2] ){  /* s2 in other cell */
        /* move rr inside of e */
        k=inside(rr,xe,rr); /* k=0: outside */
        j=get_vertex(rr);
      }
      m=n+nb*i;
      yb[m].type   = xb[n].type;
      yb[m].s1     = xb[n].s1+ns*i;
      yb[m].s2     = xb[n].s2+ns*j;
      yb[m].vec[0] = xb[n].vec[0];
      yb[m].vec[1] = xb[n].vec[1];
      yb[m].vec[2] = xb[n].vec[2];
      yb[m].r2[0]  = ((k)?0:xb[n].r2[0]); /* not correct! */
      yb[m].r2[1]  = ((k)?0:xb[n].r2[1]);
      yb[m].r2[2]  = ((k)?0:xb[n].r2[2]);
      yb[m].w      = xb[n].w;
     }
    }
    return 0;
}

int outdef(double xa[3][3], int xe[3][3], site_t *xs, bond_t *xb){
    int v,i,k,s1,s2,ns,nb;
    double rr[3];

    v=get_deti(xe);

    printf("# def-file, %s e= %d %d %d  %d %d %d  %d %d %d  v=%d\n"
           "# automaticly generated by lattice.c Jan02 JS\n",title,
           xe[0][0],xe[0][1],xe[0][2],
           xe[1][0],xe[1][1],xe[1][2],
           xe[2][0],xe[2][1],xe[2][2],v);
    printf("#===base=== 1 0 0 1 ?????\n");
    printf("#===uvec=== %.3f %.3f %.3f %.3f (unitcell)\n",
      xa[0][0],xa[0][1],
      xa[1][0],xa[1][1]);
    for(i=0;i<3;i++)printf("# %c=(%2d %2d %2d)\n",'a'+i,xe[i][0],xe[i][1],xe[i][2]);
    printf("# geometrical volume of unitcell % .6f\n",get_detd(xa));
    // better: out(prod(a,b,buf),dim)
    printf("#===edge=== %.3f %.3f  %.3f %.3f\n", /* t==e */
            xe[0][0]*xa[0][0]+xe[0][1]*xa[0][1],
            xe[0][0]*xa[1][0]+xe[0][1]*xa[1][1],
            xe[1][1]*xa[0][1]+xe[1][0]*xa[0][0],
            xe[1][1]*xa[1][1]+xe[1][0]*xa[1][0]);
    /* frame for 2dim PBC (0,0)-(0,1)-(1,1)-(1,0)-(0,0) */
    printf("#===pbcf=== 0 0  %.3f %.3f  %.3f %.3f  %.3f %.3f  0 0\n",
            xe[0][0]*xa[0][0]+xe[0][1]*xa[0][1],
            xe[0][0]*xa[1][0]+xe[0][1]*xa[1][1],
            xe[0][0]*xa[0][0]+xe[0][1]*xa[0][1]+xe[1][1]*xa[0][1]+xe[1][0]*xa[0][0],
            xe[0][0]*xa[1][0]+xe[0][1]*xa[1][1]+xe[1][1]*xa[1][1]+xe[1][0]*xa[1][0],
            xe[1][1]*xa[0][1]+xe[1][0]*xa[0][0],
            xe[1][1]*xa[1][1]+xe[1][0]*xa[1][0]);
    printf("#+++model+++\n");
    for(ns=0;ns<Ns;ns++) if (!xs[ns].type) break; /* num of sites */
    for(nb=0;nb<Nb;nb++) if (!xb[nb].type) break; /* num of bonds */
    printf("# NN=Number_of_sites Nw=Number_of_bonds Nv=Number_of_bonds(2)\n");
    printf("%d %d 0\n",ns,nb);
    printf("# s  X  Y  Z p_e p_U (s=siteindex  (X Y Z)=position  p=index of parameter)\n");
    for(i=0; i<ns; i++){
      mltply(xa,xs[i].pos,rr);
      printf(" %2d % 5.3f % 5.3f % 5.3f 8 7\n",i, rr[0], rr[1], rr[2]);
    }

    /* p_w is the index to angle for twisted boundary conditions */
    printf("# Bond s1 s2 p_t p_J p_V p_w\n");
    for(i=0; i<nb; i++){ /* all unitcells */
      k=0;
      if( xb[i].r2[0] || xb[i].r2[1] || xb[i].r2[2] ) k=1;
      s1=xb[i].s1;
      s2=xb[i].s2;
      /* Richtungsvektor fuer PBC angeben! 0..+9 = 0,n,s,e,w,ne,nw,se,sw */
      if (k) printf(" %2d %2d %2d  0 %+2d  0 %2d\n",i,s1,s2,xb[i].type,xb[i].w);
      else   printf(" %2d %2d %2d  0 % 2d  0 %2d\n",i,s1,s2,xb[i].type,xb[i].w);
    }
    printf("#---model---\n");
    return 0;
}

/* besser nur halblange bonds in alle Richtungen zeichnen? */
int outpov(double xa[3][3], int xe[3][3], site_t *xs, bond_t *xb){
    int i,j,ns,nb,s1,s2,v;
    double zoom=10.0, rr[3], r1[3], r2[3], r3[3], r4[3];
    char *farb="C"; /* declaration for color */

    v=get_deti(xe);
    printf(
"// povray - raytracer version 2.0\n"
"// clock=0..1 for stereoview\n"
"#include \"colors.inc\"\n"
"#include \"shapes.inc\"\n"
"#include \"chars.inc\"\n"
"camera {\n"
"   location  <10+clock*5, 10,-60> // right,up,away\n"
"   look_at   <20,  20,  10>\n"
"}\n"
"light_source {< 20, 80, -160> colour White}\n"
"background { color White }\n"
"union {  // Drei-Bein\n"
" object { char_X scale 0.4 translate <11,0,0> }\n"
" object { char_Y scale 0.4 translate <0,11,0> }\n"
" object { char_Z scale 0.4 translate <0,0,11> }\n"
" cone { < 0, 0, 0>,0.2  < 8,0,0>,0.2 }\n"
" cone { < 0, 0, 0>,0.2  <0, 8,0>,0.2 }\n"
" cone { < 0, 0, 0>,0.2  <0,0, 8>,0.2 }\n"
" cone { < 8, 0, 0>,0.8  <10,0,0>,0.0 }\n"
" cone { < 0, 8, 0>,0.8  <0,10,0>,0.0 }\n"
" cone { < 0, 0, 8>,0.8  <0,0,10>,0.0 }\n"
" pigment { color rgb < 0.7,0.7,0.7 > }\n"
" no_shadow\n"
" translate <-10, -5,-10>\n"
"}\n"
"#declare Site = \n"
" sphere { <0, 0, 0>, 1\n"
"   pigment { color rgb <0.4,0.4,0.8> }\n"
"   finish { ambient 0.2 diffuse 0.9 // refraction 1\n"
"   }\n"
"   no_shadow\n"
" }\n"
"#declare C0 = rgb < 0.95,0.95,0.25 >;\n"
"#declare C1 = rgb < 0.25,0.95,0.25 >;\n"
"#declare C2 = rgb < 0.95,0.25,0.25 >;\n"
"#declare C3 = rgb < 0.95,0.25,0.95 >;\n"
"#declare T0 = rgbt < 0.95,0.95,0.25,0.8 >;\n"
"#declare T1 = rgbt < 0.25,0.95,0.25,0.8 >;\n"
"#declare T2 = rgbt < 0.95,0.25,0.25,0.8 >;\n"
"#declare T3 = rgbt < 0.95,0.25,0.95,0.8 >;\n"
);

    for(ns=0;ns<Ns;ns++) if (!xs[ns].type) break; /* num of sites */
    for(nb=0;nb<Nb;nb++) if (!xb[nb].type) break; /* num of bonds */
    for(i=0; i<ns; i++){ /* all unitcells */
      mltply(xa,xs[i].pos,rr);
      printf("object { Site scale 1.0 translate <% 6.3f,% 6.3f,% 6.3f> } // s%d\n",
         zoom*rr[0], zoom*rr[1], zoom*rr[2],i);
    }

    for(i=0; i<nb; i++){ /* all unitcells */
      j=xb[i].type; s1=xb[i].s1; s2=xb[i].s2;
      add(xs[s1].pos,rr,0,rr);           mltply(xa,rr,r1); /* rr=xs+0*rr; r1=A*rr; */
      add(xs[s1].pos,xb[i].vec, 0.5,rr); mltply(xa,rr,r2);
      add(xs[s2].pos,rr,0,rr);           mltply(xa,rr,r3);
      add(xs[s2].pos,xb[i].vec,-0.5,rr); mltply(xa,rr,r4);
      farb="C";
      printf("// bond %2d ,  %2d %2d , %2d\n",i,s1,s2,xb[i].type);
#ifdef Layer1m11      /* print only [1,-1,1] layer (rgbt t=transparency) */
      if( (r1[0]-r1[1]+r1[2])*(r1[0]-r1[1]+r1[2])>0.8
       || (r3[0]-r3[1]+r3[2])*(r3[0]-r3[1]+r3[2])>0.8) farb="T";
#endif
      printf("cone {\n <% 6.3f,% 6.3f,% 6.3f>,0.5\n <% 6.3f,% 6.3f,% 6.3f>,0.5\n pigment { color %s%d }\n finish {ambient 0.2 diffuse 0.8 }\n no_shadow }\n",
                zoom*r1[0], zoom*r1[1], zoom*r1[2],
                zoom*r2[0], zoom*r2[1], zoom*r2[2], farb, j%4 );
      printf("cone {\n <% 6.3f,% 6.3f,% 6.3f>,0.5\n <% 6.3f,% 6.3f,% 6.3f>,0.5\n pigment { color %s%d }\n finish {ambient 0.2 diffuse 0.8 }\n no_shadow }\n",
                zoom*r3[0], zoom*r3[1], zoom*r3[2],
                zoom*r4[0], zoom*r4[1], zoom*r4[2], farb, j%4 );
    }
    printf("// END of model\n");
    return 0;
}

int main(int argn, char *argv[]){
  int i=1,j,v,lat=11,typ=0,vvv=1;
  while(argn>i && argv[i][0]=='-'){
    if(argv[i][1]=='l') lat=atoi(argv[i]+2);
    if(argv[i][1]=='t') typ=atoi(argv[i]+2);
    if(argv[i][1]=='d') dim=atoi(argv[i]+2);
    if(argv[i][1]=='a') vvv=0;
    i++;
  }
  if(argn== i+1 || argn== i+4 || argn== i+9){
    if( argn-i == 1) dim=1;
    if( argn-i == 4) dim=2;
    if( argn-i == 9) dim=3;
    for(j=0;j<dim*dim;j++)xe[j/dim][j%dim]=atoi(argv[j+i]);
  }
  title="unknown";
  if( dim==2 && lat==8  ){ title="til8 kagome";
    if (vvv) xa = til8_xa;
    xs = til8_xs;
    xb = til8_xb;
  } else
  if( dim==2 && lat==11 ){ title="til11";
    if (vvv) xa = til11_xa;
    xs = til11_xs;
    xb = til11_xb;
  } else
  if( dim==3 ){ title="pyrochlor (3D checker board)";
    if (vvv) xa = pyr_xa;
    xs = pyr_xs;
    xb = pyr_xb;
  } else 
  fprintf(stderr,"lattice not defined");
  
  nn=v=get_deti(xe);         /* volume of cluster */

  if( v<=0 ){ fprintf(stderr,"error: volume<=0\n");return -1; }
  vertex_init();
  ini_y( xa, xe, xs, xb );
  
  if(typ==0){ outdef( xa, xe, ys, yb ); }
  if(typ==1){ outpov( xa, xe, ys, yb ); }

  if (ys) free((void *)ys);
  if (yb) free((void *)yb);
  return 0;  
}

